import { env } from "bun";
import { SveltePlugin } from "bun-plugin-svelte";
import Tail from "bun-plugin-tailwind";
import path from "path";

const development = env.NODE_ENV !== "production";

const publicPath = development
  ? "http://localhost:3000/html/"
  : "https://userscripts.skaarup.dev/html/";

function replaceUrl(text: string): string {
  if (!development) return text;

  return text.replaceAll(
    "https://userscripts.skaarup.dev",
    "http://localhost:3000",
  );
}

const tagStart = `// ==UserScript==\n`;
const tagEnd = `// ==/UserScript==\n`;
const tagEndLen = tagEnd.length;

const globScanOptions: Bun.GlobScanOptions = {
  absolute: true,
};

const userscripts = new Bun.Glob("./src/userscripts/*/*.user.ts");
for await (const absPath of userscripts.scan(globScanOptions)) {
  const parsed = path.parse(absPath);
  console.log(`Building ${parsed.base}...`);

  const file = Bun.file(absPath);
  const fileText = await file.text();

  const bannerStart = fileText.indexOf(tagStart);
  const bannerEnd = fileText.indexOf(tagEnd) + tagEndLen;

  let banner = fileText.slice(bannerStart, bannerEnd);

  banner = replaceUrl(banner);

  await Bun.build({
    entrypoints: [absPath],
    banner,
    outdir: `./build`,
    splitting: false,
    minify: false,
    env: "PUBLIC_*",
    sourcemap: "linked",
    packages: "bundle",
    target: "browser", // Also supports "bun" or "node" for server-side components
    plugins: [
      Tail,
      SveltePlugin({
        development: false, // Enable dev features, set to false for production
      }),
    ],
    publicPath,
  });

  await swapHtmlImports(fileText, `./build/${parsed.name}.js`);
}

const styles = new Bun.Glob("./src/userscripts/*/*.user.css");
for await (const absPath of styles.scan(globScanOptions)) {
  const name = path.parse(absPath).base;

  console.log(`Building ${name}...`);

  const outFile = `./build/${name}`;

  await Bun.$`bunx @tailwindcss/cli --optimize -i ${absPath} -o ${outFile}`.quiet();

  const file = Bun.file(outFile);

  let text = await file.text();

  text = replaceUrl(text);

  file.writer().write(text);
}

async function swapHtmlImports(fileText: string, filePath: string) {
  const imports: string[] = [];

  const lines = fileText.split("\n");
  for (const line of lines) {
    if (
      !line.startsWith("import ") ||
      (!line.endsWith('.html";') && !line.endsWith(".html';"))
    )
      continue;

    // Extract the import path between the quotes
    const match = line.match(/['"]([^'"]+)['"]/);
    if (!match) continue;

    const importPath = match[1];
    // Check if the import path is a .html file
    if (!importPath) continue;
    imports.push(path.parse(importPath).name);
  }

  const outFile = Bun.file(filePath);
  const text = await outFile.text();

  const startIdx = text.indexOf(tagStart);
  const endIdx = text.indexOf(tagEnd);
  const banner = text.slice(startIdx, endIdx);

  let js = text.slice(endIdx);

  const htmlFiles = new Bun.Glob("./build/*.html");
  for await (const filePath of htmlFiles.scan(globScanOptions)) {
    const parsed = path.parse(filePath);
    const fileName = parsed.base;
    if (!imports.some((x) => fileName.startsWith(x))) continue;

    const file = Bun.file(filePath);
    const html = await file.text();

    js = js.replace(`${publicPath}${fileName}`, btoa(html));
  }

  outFile.writer().write(`${banner}${js}`);
}
