import { env } from "bun";
import homepage from "./index.html";
import robotsTxt from "./static/robots.txt";
import sitemapTxt from "./static/sitemap.txt";
// @ts-ignore ts2307
import icon from "./static/favicon.png" with { type: "file" };
import path from "path";

const favicon = await Bun.file(icon).bytes();

let htmlFiles: Set<string>;
let scriptFiles: Set<string>;
let cssFiles: Set<string>;

const development = env.NODE_ENV !== "production";

const globScanOptions: Bun.GlobScanOptions = {
  absolute: true,
};

async function load() {
  await Bun.$`bun run src/build.ts`;
  const tempHtml: Set<string> = new Set();
  const userHtml = new Bun.Glob("./build/*.html");
  for await (const filePath of userHtml.scan(globScanOptions)) {
    tempHtml.add(path.parse(filePath).base);
  }
  htmlFiles = tempHtml;

  const tempScripts: Set<string> = new Set();
  const userJs = new Bun.Glob("./build/*.user.js");
  for await (const filePath of userJs.scan(globScanOptions)) {
    tempScripts.add(path.parse(filePath).base);
  }
  scriptFiles = tempScripts;

  const tempStyles: Set<string> = new Set();
  const css = new Bun.Glob("./build/*.user.css");
  for await (const filePath of css.scan(globScanOptions)) {
    tempStyles.add(path.parse(filePath).base);
  }
  cssFiles = tempStyles;
}
await load();

Bun.serve({
  routes: {
    "/": homepage,
    "/robots.txt": new Response(robotsTxt, {
      headers: { "Content-Type": "text/plain" },
    }),
    "/sitemap.txt": new Response(sitemapTxt, {
      headers: { "Content-Type": "text/plain" },
    }),
    "/favicon.ico": new Response(favicon, {
      headers: { "Content-Type": "image/png" },
    }),
    "/health": new Response("OK"),
    "/html/:html": async (req) => {
      const html = req.params.html.toLocaleLowerCase();

      if (!htmlFiles.has(html)) {
        return new Response("Not found", { status: 404 });
      }

      const file = Bun.file(`./build/${html}`);

      return new Response(file, {
        headers: { "Content-Type": "text/css" },
      });
    },
    "/scripts/:script": async (req) => {
      const script = req.params.script.toLocaleLowerCase();

      if (!scriptFiles.has(script)) {
        return new Response("Not found", { status: 404 });
      }

      const file = Bun.file(`./build/${script}`);

      return new Response(file, {
        headers: { "Content-Type": "application/javascript" },
      });
    },
    "/styles/:style": async (req) => {
      const style = req.params.style.toLocaleLowerCase();

      if (!cssFiles.has(style)) {
        return new Response("Not found", { status: 404 });
      }

      const file = Bun.file(`./build/${style}`);

      return new Response(file, {
        headers: { "Content-Type": "text/css" },
      });
    },
  },
  development,
  // async fetch(req, server) {
  //   return new Response("Not found", { status: 404 });
  // },
});

if (development) {
  setInterval(async () => {
    try {
      await load();
    } catch (e) {
      console.error(e);
    }
  }, 2500);
}
